<?php

namespace Scriptor\Modules\Quill;

use Imanager\TemplateParser;
use Scriptor\Core\Module;

/**
 * Quill Class - Scriptor Extension Module.
 * 
 * Replaces Scriptor's default Markdown editor with Quill. 
 * More information under: https://quilljs.com
 * 
 */
class Quill extends Module
{
	/**
	 * Module version 
	 * 
	 * Use ~~ echo Quill::VERSION; ~~~
	 * to get the current module version
	 */
	const VERSION = '1.5.1';

	private static $initialized = false;

	public static function moduleInfo() : array
	{
		return [
			'name' => 'Quill',
			'menu' => '',
            'position' => 100,
            'active' => true,
            'auth' => true,
            'autoinit' => true,
            'display_type' => [],
            'description' => 'Quill Content Editor is an extension module for Scriptor CMS. It provides a rich and intuitive content editing experience. <a href="https://scriptor-cms.info/extensions/extensions-modules/quillcontenteditor/">More information</a>',
			'version' => self::VERSION,
			'author' => 'Bigin',
			'author_website' => 'https://ehret-studio.com'
		];
	}

	public static function moduleHooks() : array
	{
		return [
			'Pages::beforeRenderEditorPage' => [
				[
					'module' => 'Quill',
					'method' => 'beforeRenderPageEditor'
				]
			],
			'Pages::afterRenderEditorContentField' => [
				[
					'module' => 'Quill',
					'method' => 'renderEditorContentField'
				]
			],
			'Pages::afterCheckAction' => [
				[
					'module' => 'Quill',
					'method' => 'checkAction'
				]
			],
			'Pages::afterRenderEditorPage' => [
				[
					'module' => 'Quill',
					'method' => 'createImageSelector'
				]
			],
			'Pages::beforeSavePage' => [
				[
					'module' => 'Quill',
					'method' => 'prepareContentLinks'
				]
			],
			'Pages::afterSavePage' => [
				[
					'module' => 'Quill',
					'method' => 'replaceTimestampPlaceholders'
				]
			]
		];
	}

	/* public function init()
	{
		if (!self::$initialized) {
			parent::init();
			self::$initialized = true;
		}
	} */

	/**
	 * Runs when the editor view is called and 
	 * loads some CSS resources within the header.
	 * 
	 * @param object $event
	 */
	public function beforeRenderPageEditor($event)
	{
		$dir = dirname($this->siteUrl);

		$this->addResource(
			'link', ['rel' => 'stylesheet',
				'href' => $dir.'/site/modules/Quill/quill/quill.snow.css'], 'header'
		);
		$this->addResource(
			'link', ['rel' => 'stylesheet',
				'href' => $dir.'/site/modules/Quill/css/styles.css'], 'header'
		);
		$this->addResource(
			'link', ['rel' => 'stylesheet',
				'href' => $dir.'/site/modules/Quill/highlight/styles/monokai-sublime.css'], 'header'
		);
	}

	/**
	 * This method replaces the default Markup editor 
	 * with DOM elements that will contain Quill.
	 * 
	 * @param object $event
	 */
	public function renderEditorContentField($event)
	{
		// Get current page
		$page = $event->object->page;
		// Set post param for preparing the image links 
		$content = '';
		if(isset($page->content)) {
			$content = $this->parseContent($event->object);
		}
		ob_start(); ?>
		<div class="form-control">
			<label class="required" for="quill-editor"><?php 
				echo $this->i18n['content_label']; ?></label>
			<div id="quill-editor">
				<?php echo $content; ?>
			</div>
			<textarea id="quill-content" name="content"><?php 
				echo isset($page->content) ? $page->content : ''; ?></textarea>
		</div>
		<?php 
		// Replace default editor with Quill
		$event->return = ob_get_clean();
	}

	/**
	 * Prepares content for display in the editor. 
	 * Replaces placeholders and decodes HTML entities.
	 * 
	 * @param object $object - Scriptor\Page instance  
	 */
	private function parseContent($object)
	{
		$templateParser = new TemplateParser();

		$imgPath = '';
		$field = $object->pages->getField('name=images');
		$pageId = isset($object->page->id) ? $object->page->id : null;
		if($pageId && $field && $object->pages) {
			$imgPath = "$pageId.{$object->pages->id}.$field->id/";
		} else {
			$imgPath = ".tmp_{$object->input->post->timestamp_images}_{$object->pages->id}.$field->id/";
		}
		
		$result = $templateParser->render($object->page->content, [
			'BASE_URL' => dirname($object->siteUrl).'/',
			'UPLOADS_URL' => dirname($object->siteUrl).'/data/uploads/',
			'IMAGES_URL' => dirname($object->siteUrl)."/data/uploads/$imgPath",
		]);

		return html_entity_decode($result);
	}

	/**
	 * Is called after the method for displaying the page editor 
	 * was executed. This method prepares modal markup for displaying 
	 * the ImageSelector and also loads the required JavaScript 
	 * libraries. 
	 * 
	 * @param object $event
	 */
	public function createImageSelector($event)
	{
		$dir = dirname($this->siteUrl);
		ob_start(); ?>
		<!-- The Modal -->
		<div id="images-modal" class="modal">
			<!-- Modal content -->
			<div class="modal-content">
				<span class="modal-close">&times;</span>
				<div id="modal-image-selector"></div>
			</div>
		</div>
		<!-- Initialize Quill editor -->
		<script src="<?php 
			echo $dir.'/site/modules/Quill/highlight/highlight.pack.js'; ?>"></script>
		<script src="<?php 
			echo $dir.'/site/modules/Quill/quill/quill.min.js'; ?>"></script>
		<script src="<?php 
			echo $dir.'/site/modules/Quill/scripts/quill-config.js'; ?>"></script>
		<?php
		$event->return .= ob_get_clean();
	}

	/**
	 * Checks if an image selector was called 
	 * via Ajax and, if so, executes the function 
	 * to prepare the selector view for page images
	 * 
	 * @param object $event
	 */
	public function checkAction($event)
	{
		if($event->object->input->post->action == 'image-selector') {			
			$rows = $this->buildImageSelector($event->object);
			$this->sendJsonResponse(['rows' => $rows]);
		}
	}

	/**
	 * This method prepares the rows with images 
	 * for the ImageSelector.
	 * 
	 * @param object $instance
	 */
	private function buildImageSelector($instance)
	{
		$pages = $instance->pages;
		$page = $instance->page;
		$input = $instance->input;

		$imgPath = '';
        $field = $pages->getField('name=images');
        $pageId = isset($page->id) ? $page->id : null;
        if($pageId && $field && $pages) {
			$imgPath = $pages->id.".$pageId.$field->id/";
        } else {
			$imgPath = ".tmp_{$input->post->timestamp_images}_{$pages->id}.$field->id/";
        }
		
		$images = $this->getImagesArray($imgPath);
		if(empty($images)) {
			return '<p>'.$this->i18n['image_selector_empty'].'</p>';
		}

		$rows = '';
		foreach($images as $key => $image) {
			$rows .= '<div class="row">'.
					 '	<div class="col"><a class="image-select" href="'.$image['url']
					 	.'"><img height="100" src="'.$image['url'].'" alt="'.$image['name'].'"></a></div>'.
				     '</div>';
		}
		return $rows;
	}

	/**
	 * Sends Json response and terminates the script execution.
	 *
	 * @param string|array $data - Data to send
	 */
	private function sendJsonResponse($data)
	{
		header('Content-type: application/json; charset=utf-8');
		echo json_encode($data);
		exit;
	}

	/**
	 * Returns all uploaded page images as an array
	 * 
	 * @param string $path - Path to the page image dir
	 */
	private function getImagesArray($path)
	{
		$imagesArray = [];
		$i = 0;
		foreach(glob(IM_UPLOADPATH.$path.'*.{jpg,jpeg,png,gif}', GLOB_BRACE) as $file) {
			$base = basename($file);
			$imagesArray[$i]['url'] = dirname($this->siteUrl).'/data/uploads/'.$path.$base;
			$imagesArray[$i]['name'] = $base;
			$i++;
		}
		return $imagesArray;
	}

	/**
	 * This method prepares content of the 
	 * page before it is saved.
	 * 
	 * It detects a part of the absolute URL https://your-site/data/uploads/... 
	 * and replaces it with a placeholder [[UPLOADS_URL]]. his placeholder is 
	 * replaced dynamically when the content is loaded.
	 * 
	 * @param object $event - Hook event
	 * 
	 */
	public function prepareContentLinks($event)
	{
		$post = $this->input->post;
		$field = $event->object->pages->getField('name=images');
		if(!isset($event->object->page->id) && $post->timestamp_images) {
			$this->input->whitelist->uploadTempDir = 
				".tmp_{$post->timestamp_images}_{$event->object->pages->id}.$field->id/";
		}
		
		// Get the variables we use locally
		$content = str_replace(dirname($this->siteUrl).'/data/uploads/', 
			'[[UPLOADS_URL]]', $post->content
		);
		$this->input->post->content = $content;
	}

	/**
	 * Replace temporary image folder with a persistent one. 
	 * 
	 * Only applies if an image has been selected before the 
	 * page has been saved for the first time.
	 * 
	 * @param object $event - Hook event
	 */
	public function replaceTimestampPlaceholders($event)
	{
		// Check if saving the page was successful, if not just leave
		if(empty($this->input->whitelist->uploadTempDir) || !$event->return) return;
		// Get the variables we use locally
		$page = $event->object->page;
		$pages = $event->object->pages;

		if(isset($page->id) && isset($page->content)) {
			$field = $pages->getField('name=images');
			$imgPath = $pages->id.".$page->id.$field->id/";

			$content = str_replace($this->input->whitelist->uploadTempDir, 
				$imgPath, $page->content
			);

			$page->set('content', $content)
				->save();
		}
	}
}
